from api_hunt.core import run_scan_git,run_scan_local
from api_hunt.add_delete import display,delete_data_from_custom,add_data_to_custom
from api_hunt.get_repattern_ai import set_google_api_key,anonymize_digits,get_pattern_gemini
from api_hunt.patterns import API_KEY_PATTERNS, SCAN_EXTENSIONS, IGNORE_PATTERNS

import asyncio

import argparse

def main():
    parser = argparse.ArgumentParser(description='Helps to find API keys in the git staged files i.e (when you git add files) or can search a single local file.')
    parser.add_argument('-d', '--display',action='store_true', help="Display's the custom patterns stored")
    parser.add_argument('-r','--remove',action="store",nargs=1,type=str,help="Remove a stored pattern takes 'key_name' as argument e.g -> hunt -r 'key_name'")
    parser.add_argument('-a','--add',action="store",nargs=2,type=str,help="adds a key and pattern to custom patterns file takes 'key_name' and 'pattern' as arguments e.g -> hunt -a 'key_name' 'pattern'")
    parser.add_argument('-n','--name',action="store",nargs=1,type=str,help="Scans a single file for API key's takes filepath as argument e.g -> hunt -n 'file_path'")
    parser.add_argument('-c','--config',action="store",nargs=1,type=str,help="Takes an API key as argument to configure gemini to be used to get custom regex pattern for your API key e.g -> hunt -c 'API_KEY'")
    parser.add_argument('-re','--regex',action="store",nargs=2,type=str,help="Takes a key_name and API key as arguments to give regex of it which will be added automatically to custom patterns folder to be used to search for API key's  e.g -> hunt -re 'key_name' 'API_key' ")
    parser.add_argument('-v','--verbose',action="store_true",help="Gives verbose result i.e also includes the Matched Pattern in the Output")
    args = parser.parse_args()

    if args.display:
        main_display()
    
    elif args.remove:
        print(f"removing key {args.remove[0]}")
        remove(args.remove[0])
    
    elif args.add:
        print(f"adding key and pattern")
        add_data_to_custom(args.add[0],args.add[1])
    
    elif args.name:
        print(f"Scanning file {args.name[0]}")
        if args.verbose:
            hunt_local(args.name[0],verbose=True)
        else:
            hunt_local(args.name[0],verbose=False)

    elif args.config:
        print("setting api key")
        set_google_api_key(args.config[0])
    
    elif args.regex:
        print("getting the regex pattern")
        get_regex_add(args.regex[0],args.regex[1])
    
    elif args.verbose:
        hunt(verbose=True)

    else:
        hunt(verbose=False)


def hunt(verbose):
    print("Scanning for secrets...")
    asyncio.run(run_scan_git(verbose))


def main_display():
    display()

def remove(key_name):
    delete_data_from_custom(key_name)

def hunt_local(file_path,verbose):
    asyncio.run(run_scan_local(file_path,verbose))


def get_regex_add(key_name,api_key):
    anon_api_key = anonymize_digits(api_key)
    res = get_pattern_gemini(key_name,anon_api_key)
    if res:
        add_data_to_custom(res["key_name"],res["pattern"])
    else:
        print("error getting regex pattern and adding to custom json")


import subprocess
import asyncio
import os
from pathlib import Path
import re
import json
from api_hunt.patterns import API_KEY_PATTERNS, SCAN_EXTENSIONS, IGNORE_PATTERNS

def is_git_directory(path='.'):
    try:
        result = subprocess.run(
            ['git', '-C', path, 'rev-parse', '--is-inside-work-tree'],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=True,
            check=True
        )
        return result.stdout.strip() == 'true'
    except subprocess.CalledProcessError:
        return False

def get_staged_files():
    try:
        result = subprocess.run(["git","diff","--cached","--name-only"],
                                text=True,capture_output=True)
        
        if result.returncode != 0:
            print(f"error running subprocess {result.stderr}")
            return []
        else:
            return [f.strip() for f in result.stdout.splitlines() if f.strip()]
        
    except subprocess.CalledProcessError as e:
            print(f"Error running git diff: {e.stderr}")
            return []
    except subprocess.SubprocessError as e:
            print(f"Subprocess error: {e}")
            return []





def should_ignore(file_path):
    for pattern in IGNORE_PATTERNS:    
        if re.search(pattern,file_path):
            return True
    return False


def should_search(file_path):
    if should_ignore(file_path):
        return False
    _,ext = os.path.splitext(file_path)
    if ext.lower() in SCAN_EXTENSIONS:
        return True
    else:
        return False

async def get_file_content(file_path):

    try:
        proc = await asyncio.create_subprocess_exec("git","show",f":{file_path}",
                                stdout=asyncio.subprocess.PIPE,
                                stderr=asyncio.subprocess.PIPE)
        
        stdout,stderr = await proc.communicate()
        
        if proc.returncode != 0:
            print(f"Error running git show: {stderr.decode()}")
            return ""        
        
        content = stdout.decode()
        return content
    
    except Exception as e:
        print(f"error getting file contents {e}")
        return ""
    



async def scan_content(content:str,file_path:str):

    try:
        custom_pattern_path = Path.home() / "api_hunt_envs" / "custom_pattern.json"

        if custom_pattern_path.exists() and os.stat(custom_pattern_path).st_size !=0:
            with open(custom_pattern_path,"r") as f:
                data = json.load(f)

            API_KEY_PATTERNS_COMBINED = API_KEY_PATTERNS + [d["pattern"] for d in data]
        else:
            API_KEY_PATTERNS_COMBINED = API_KEY_PATTERNS

        findings = []

        for i,line in enumerate(content.splitlines(),1):
            for pattern in API_KEY_PATTERNS_COMBINED:
                matches = re.finditer(pattern,line,re.IGNORECASE)
                if not matches:
                    continue
                for match in matches:
                    findings.append({
                            "file_name":file_path,
                            "match":match.group(0),
                            "line":i,
                            "context": line.strip()
                        
                        })
        return findings
    
    except Exception as e:
        return f"error scanning files error {e}"
    


async def process(file):
    content = await get_file_content(file)
    return await scan_content(content,file)


async def run_scan_local(file_path,verbose=False):
    
    try:
        YELLOW_BG = "\033[30;43m"
        GREEN_BG  = "\033[30;42m"
        RED_BG    = "\033[30;41m"
        RESET     = "\033[0m"

        with open(file_path,"r") as file:
            content = file.read()
        findings = await scan_content(content,file_path)
   
        unique_logs = []
        seen = set()
        for res in findings:
            key = (res.get("file_name",''),res.get("line",''),res.get("match",''))
            check = (key[0],key[1])
            if check not in seen:
                seen.add(check)
                unique_logs.append(key)

        if not unique_logs:
            print("No Possible key found.")
        else:
            for logs in unique_logs:
                    if verbose:
                        print(
                            f"found possible API key in file {YELLOW_BG}{logs[0]}{RESET}, "
                            f"{GREEN_BG} line : {logs[1]}{RESET}, "
                            f"matched pattern : {RED_BG}{logs[2]}{RESET}"
                        )
                    else:
                        print(
                            f"found possible API key in file {YELLOW_BG}{logs[0]}{RESET}, "
                            f"{GREEN_BG} line : {logs[1]}{RESET}, "
                        )
      

    except FileNotFoundError as e:
        print(f"File not found error {e}")
        return
    except Exception as e:
        print(f"Error scanning local file {file_path}, error {e}")
        return




async def run_scan_git(verbose=False):
    if is_git_directory():
        files = get_staged_files()

        YELLOW_BG = "\033[30;43m"
        GREEN_BG  = "\033[30;42m"
        RED_BG    = "\033[30;41m"
        RESET     = "\033[0m"

        if isinstance(files,list) and len(files)>0:
            tasks = [process(file) for file in files
                    if should_search(file)]
            
            results = await asyncio.gather(*tasks)

            flat_results = [log for r in results for log in r]

            unique_logs = []
            seen = set()
            for res in flat_results:
                key = (res.get("file_name",''),res.get("line",''),res.get("match",''))
                check = (key[0],key[1])
                if check not in seen:
                    seen.add(check)
                    unique_logs.append(key)

            if not unique_logs:
                print("No Possible key found.")
            else:
                for logs in unique_logs:
                    if verbose:
                        print(
                            f"found possible API key in file {YELLOW_BG}{logs[0]}{RESET}, "
                            f"{GREEN_BG} line : {logs[1]}{RESET}, "
                            f"matched pattern : {RED_BG}{logs[2]}{RESET}"
                        )
                    else:
                        print(
                            f"found possible API key in file {YELLOW_BG}{logs[0]}{RESET}, "
                            f"{GREEN_BG} line : {logs[1]}{RESET}, "
                        )

        elif len(files)==0:
            print("no files found in index please git add to add files")
   
    else:
        print("No git found please intialize a git Repo")




from google.genai import Client
import random
import re
from pydantic import BaseModel
import dotenv
import os
import ast
from dotenv import load_dotenv
from pathlib import Path

class ResponseFormat(BaseModel):
    key_name:str
    pattern:str

env_dir = Path.home() / "api_hunt_envs"
env_dir.mkdir(parents=True,exist_ok=True)


def anonymize_digits(api_key):
    return re.sub(r'\d', lambda _: str(random.randint(0, 9)), api_key)

def set_google_api_key(api_key):

    try:
        dotenv_path = env_dir / ".api_hunt_env"

        if not os.path.exists(dotenv_path):
            open(dotenv_path, 'a').close() 
            print(".env file not found — created new one.")

        dotenv.load_dotenv(dotenv_path)
        if dotenv.set_key(dotenv_path,"GOOGLE_API_KEY",api_key)[0]:
            print("Success loading api key")
            return
        else:
            print("failed to load api key")
            return
    
    except Exception as e:
        print(f"error setting api key error- {e}")


def get_pattern_gemini(name,anon_api_key)->dict:

    try:
        dotenv_path = env_dir / ".api_hunt_env"
        if os.path.exists(dotenv_path):
            load_dotenv(dotenv_path)
        else:
            print("error getting dotenv path check if gemini api key is configured")
            return

        gemini_api_key = os.getenv("GOOGLE_API_KEY")
        if not gemini_api_key:
            print("error getting gemini api key please check if gemini api key is configured")
            return

        client = Client(api_key=gemini_api_key)

        sys_prompt = f"""
        You are an expert in generating regex patterns that match API keys.
        Given an API key, return a JSON object with:
        if key_name: {name} if given by user then use that else 
        - "key_name": a guessed type or label in underscores (e.g. "bearer","aws", "openai")
        - "pattern": a regex that matches keys of the same kind

        Respond only in JSON.

        """
        response = client.models.generate_content(model="gemini-2.0-flash",
                                                contents=sys_prompt + str(anon_api_key),
                                                config={
                                                    "response_mime_type":"application/json",
                                                    "response_schema":ResponseFormat,
                                                })

        if isinstance(response.text,str):
            response_dict = ast.literal_eval(response.text)
            if response_dict:
                print(f"found pattern {response_dict.get('pattern','')}")
                return {"key_name":response_dict.get('key_name',''),
                        "pattern":response_dict.get('pattern','')}
            else:
                print("error getting pattern try again")
                return
            
    except Exception as e:
        print(f"error getting regex pattern from gemini error -{e}")



from setuptools import setup, find_packages

setup(
    name='apikey_hunter',
    version='0.1.3',
    packages=find_packages(),
    install_requires=["google-genai","python-dotenv","pydantic"],
    entry_points={
        'console_scripts': [
            'hunt=api_hunt.cli:main',  
        ],
    },
    author='Joe Mama',
    description='Scan staged Git files for secrets like API keys.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    classifiers=[
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.7',
)



import os
import json
from pathlib import Path

file_path = Path.home() / "api_hunt_envs" / "custom_pattern.json"

file_path.parent.mkdir(parents=True, exist_ok=True)

if file_path.parent.exists():
    FILE_PATH = file_path
else:
    FILE_PATH = f"{Path.home()}/custom_pattern.json"

def display():
    if not os.path.exists(FILE_PATH) or os.stat(FILE_PATH).st_size <= 2:
        
        print("File is empty or not found")
        return
    
    with open(FILE_PATH,"r") as file:
        data = json.load(file)

    for d in data:
        print(f"key_name: {d['key_name']}, pattern: {d['pattern']}")

def delete_data_from_custom(key_name):

    if not os.path.exists(FILE_PATH) or os.stat(FILE_PATH).st_size <= 2:
        
        print("File is empty or not found")
        return
    with open(FILE_PATH,'r') as file:
        data = json.load(file)

    deleted = False
    for i in range(len(data)-1,-1,-1):
        if key_name.lower() in data[i].get("key_name","").lower():
            del data[i]
            deleted = True

    if deleted: 
        with open(FILE_PATH,"w") as file:
            json.dump(data,file,indent=4)
        print(f"deleted key {key_name}")
        return
    else:
        print("key not found")
        return


def add_data_to_custom(key:str,pattern:str):

    if not key and not pattern:
        print("No Key and Pattern provided as input")
        return
    
    if not os.path.exists(FILE_PATH) or os.stat(FILE_PATH).st_size == 0:
        with open(FILE_PATH, 'w') as f:
            json.dump([], f)

    data = {"key_name":key,"pattern":pattern}

    with open(FILE_PATH,"r") as file:
        content = json.load(file)

    content.append(data)

    with open(FILE_PATH,"w") as file:
        json.dump(content,file,indent=4)
    print(f"data added to file {data}")